import java.io.*;

import unicamp.ic.mc336.definitions.*;
import java.util.Vector;

public class CLInterpreter {

    public void iterate(PlayerInterface player) {
        BufferedReader br = new BufferedReader(new InputStreamReader(System.in));


        // Verifica se extendem as classes
        if ((!(player instanceof AbstractCatcherPlayer)) && (!(player instanceof AbstractCatPlayer))) {
            System.err.println("You must extend AbstractCatPlayer or AbstractCatcherPlayer");
            System.exit(1);
        }
        String startStr = "";
        try {
            startStr = br.readLine();
	    exit(startStr);
        } catch (IOException ioe) {
            System.err.println("IO error trying to read start!");
            System.exit(1);
        }

        start(player, startStr);

        String movement = "";
        while (true) {
            try {
                movement = br.readLine();
		exit(movement);
            } catch (IOException ioe) {
                System.err.println("IO error trying to read move!");
                System.exit(1);
            }
            move(player, movement);
        }
    }

    private void exit(String move){
	if (move.equals("quit"))
	    System.exit(1);
    }

    private void move(PlayerInterface player, String move) {
        try {
            String moveArray[] = move.split(",");
            if (moveArray.length != 2) {
                System.err.println("Invalid Movement " + move);
                System.exit(1);
            }

            int j = Integer.parseInt(moveArray[0].substring(1,moveArray[0].length()));
            int i = Integer.parseInt(moveArray[1].substring(0,moveArray[1].length()-1));

            Position newMove = player.move(new Position(j, i));
            System.out.println("("+newMove.toString()+")");

        } catch (NumberFormatException nfe) {
            System.err.println("Invalid Movement " + move);
            System.exit(1);
        }
    }

    private void start(PlayerInterface player, String str) {
        String startVet[] = str.split(" ");
        if ((startVet.length < 1) || (startVet.length > 2)) {
            System.err.println("Invalid Start Board or catPosition.");
	    System.err.println("It was expected (jc,jc) (jb1,ib1;jb2,ib2;jbn,ibn)");
	    System.err.println("Where:");
	    System.err.println("     (jc,ic) are cat coordinates");
	    System.err.println("     (jb1,jb1;jb2,jb2;jbn,jbn) are blocks list");
            System.exit(1);
        }

        Vector<Position> blockedList;
        Position catPosition = getCat(startVet[0]);
        if (startVet.length == 2) {
            blockedList = getBoard(startVet[1]);
        } else {
            blockedList = new Vector<Position>();
        }

        player.start(catPosition, blockedList);
        System.out.println("ok");
    }

    private Vector<Position> getBoard(String str) {
        Vector<Position> startBlockList = new Vector<Position>();
	String newStr = str.substring(1,str.length()-1);
        String moves[] = newStr.split(";");
        for (int i = 0; i <
                moves.length; i++) {
            String ji[] = moves[i].split(",");
            int column, line;
            try {
                column = Integer.parseInt(ji[0]);
                line   = Integer.parseInt(ji[1]);
                startBlockList.add(new Position(column, line));
            } catch (Exception e) {
                System.err.println("Invalid board: "+str);
		System.err.println("You should enter the borar in (jb1,ib1;jb2,ib2;jbn,ibn) format");
		System.err.println("Where:");
		System.err.println("     (jb1,jb1;jb2,jb2;jbn,jbn) are blocks list");
		System.exit(1);
            }

        }
	//System.err.println(startBlockList.toString());
        return startBlockList;
    }

    private Position getCat(String str) {
        Position catPosition = new Position(6, 6);
        String ji[] = str.split(",");
        int column, line;
        try {
            int j = Integer.parseInt(ji[0].substring(1,ji[0].length()));
            int i = Integer.parseInt(ji[1].substring(0,ji[1].length()-1));

            catPosition = new Position(j, i);
        } catch (Exception e) {
            System.err.println("Invalid cat position: "+str);
	    System.err.println("You should enter the cat position in (jc,ic) format");
            System.err.println("Where:");
            System.err.println("     (jc,ic) are cat coordinates");
	    System.exit(1);
        }

        return catPosition;
    }

    public static void main(String args[]) {
        if (args.length != 2) {
            System.out.println("usage: java CLInterpreter (catcher|cat) filename");
            return;
        }

        if ((!args[0].equals("catcher")) && (!args[0].equals("cat"))) {
            System.out.println("usage: java CLInterpreter (catcher|cat) filename");
            return;
        }

        try {
            CLInterpreter clInterpreter = new CLInterpreter();
            String playerPath = args[1];
            ContainerInterface container = (ContainerInterface) Class.forName(playerPath).newInstance();
            PlayerInterface player;

            if (args[0].equals("catcher")) {
                player = container.getCatcherPlayer();
            } else {
                player = container.getCatPlayer();
            }

            clInterpreter.iterate(player);
        } catch (ClassNotFoundException e) {
            System.err.println("The player " + args[1] + " was not found.");
        } catch (InstantiationException e) {
            System.err.println("I can not instantiate player");
        } catch (ClassCastException e) {
            System.err.println("You must implement ContainerInterface");
        } catch (Exception e) {
            //System.err.println("[ERROR] An error occurred with player " + args[1] + ": " + e.getMessage());
        }
    }
}
