package unicamp.ic.mc336.definitions;

/**
 * Classe que deve ser extendida pelo aluno para criar a sua versao do gato. <br>
 * Esta classe implementa como 'final' o metodo move definido em PlayerInterface. 
 * Dessa forma, o aluno nao podera sobrescreve-lo, sendo necessario implementar os
 * outros metodos definidos como abstratos para alcancar o comportamento desejado do gato. <br>
 * Segue a implementacao final do metodo move:<br>
 <pre>
      public final Position move(Position LastMove) {<br>
          addBlock(LastMove);<br>
          Position myMove = calcMove();<br>
          while (!validMove(myMove)) {<br>
                  myMove = calcMove();<br>
          }<br>
          defineCatPosition(myMove);<br>
          return myMove;<br>
      }
 </pre>
 *  
 * @author Andre, Ulisses e Zanoni
 */
public abstract class AbstractCatPlayer extends AbstractPlayer {
    /**
     * Metodo invocado pelo campeonato para solicitar jogada. 
     * Observe que o limite de tempo desse metodo e de 1 segundo. Segue a implementacao
     * final do metodo: <br>
 <pre>
      public final Position move(Position LastMove) {<br>
          addBlock(LastMove);<br>
          Position myMove = calcMove();<br>
          while (!validMove(myMove)) {<br>
                  myMove = calcMove();<br>
          }<br>
          defineCatPosition(myMove);<br>
          return myMove;<br>
      }
 </pre>
     * @param LastMove Ultima jogada realizada pelo oponente
     * @return Jogada realizada em resposta ao oponente
     */
    public final Position move(Position LastMove) {
        addBlock(LastMove);
        Position myMove = calcMove();
        while (!validMove(myMove)) {
            myMove = calcMove();
        }
        defineCatPosition(myMove);
        return myMove;
    }

    /**
     * Metodo que indica para onde o gato deve se mover. O movimento repassado ao 
     * campeonato e fornecido por este metodo. e importante que o aluno 
     * defina o que e um movimento valido reescrevendo o metodo validMove.
     * @return Uma nova posicao para o gato.
     */
    public abstract Position calcMove();

    /**
     * Por meio deste metodo, o aluno deve definir o que considera um movimento 
     * valido para o gato.
     * @param myMove Movimento a ser validado.
     * @return Se o movimento e valido ou nao.
     */
    public abstract boolean validMove(Position myMove);
}
