package unicamp.ic.mc336.definitions;

import java.util.Vector;

/**
 * Classe abstrata que define o metodo start. A definicao do metodo start 
 * necessita de dois outros metodos: defineCatPosition e defineBlockList.<br>
 * Segue a implementacao do metodo start:<br>
<pre>
public void start(Position catPosition, Vector<Position> blockList) {<br>
defineCatPosition(catPosition);<br>
defineBlockList(blockList);<br>
}</pre> <br>
 * @author Andre, Ulisses e Zanoni
 */
public abstract class AbstractPlayer implements PlayerInterface {
    protected Vector<Position> blockList;
    protected Position catPosition;
    /**
     * Metodo invocado apenas uma vez pelo campeonato no inicio do jogo. 
     * A implementacao do metodo e:<br>
    <pre>
    public void start(Position catPosition, Vector<Position> blockList) {<br>
    defineCatPosition(catPosition);<br>
    defineBlockList(blockList);<br>
    }</pre> <br>
     * @param catPosition Posicao inicial do gato
     * @param blockList Lista de posicoes bloqueadas no inicio do jogo
     */
    public void start(Position catPosition, Vector<Position> blockList) {
        defineCatPosition(catPosition);
        defineBlockList(blockList);
    }

    /**
     * Metodo utilizado pelo aluno para receber do campeonato a posicao do 
     * gato. e importante que o aluno receba o gato e armazene em alguma 
     * variavel.
     * @param position Posicao do gato.
     */
    public void defineCatPosition(Position position){
        this.catPosition = position;
    }

    /**
     * Metodo utilizado pelo aluno para receber a configuracao inicial do 
     * tabuleiro. O tabuleiro e passado para o aluno como um vetor de 
     * posicoes. Entretanto, o aluno esta livre para criar internamente outra 
     * estrutura de dados.
     * @param blockList Configuracao inicial do tabuleiro.
     */
    public void defineBlockList(Vector<Position> blockList){
        this.blockList = blockList;
    }

    /**
     * Metodo pelo qual o aluno recebe uma nova posicao bloqueada. 
     * @param newBlock Nova posicao bloqueada
     */
    public void addBlock(Position newBlock){
        this.blockList.add(newBlock);
    }
}
